﻿//Kod kreskowy Code 93
//(c) 2019 by Tomasz Lubinski
//www.algorytm.org

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;

namespace Barcodes.Code93
{
    public partial class Form1 : Form
    {
        private readonly static char special_1 = '\u0080'; // ($)
        private readonly static char special_2 = '\u0081'; // (%)
        private readonly static char special_3 = '\u0082'; // (/)
        private readonly static char special_4 = '\u0083'; // (+)
        private readonly static char[] code93signs = 
        {
            '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
            'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
            'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
            'U', 'V', 'W', 'X', 'Y', 'Z', '-', '.', ' ', '$',
            '/', '+', '%', special_1, special_2, special_3, special_4
        };

        private readonly static int[,] code93bars = 
        {
            {1, 0, 0, 0, 1, 0, 1, 0, 0},
            {1, 0, 1, 0, 0, 1, 0, 0, 0},
            {1, 0, 1, 0, 0, 0, 1, 0, 0},
            {1, 0, 1, 0, 0, 0, 0, 1, 0},
            {1, 0, 0, 1, 0, 1, 0, 0, 0},
            {1, 0, 0, 1, 0, 0, 1, 0, 0},
            {1, 0, 0, 1, 0, 0, 0, 1, 0},
            {1, 0, 1, 0, 1, 0, 0, 0, 0},
            {1, 0, 0, 0, 1, 0, 0, 1, 0},
            {1, 0, 0, 0, 0, 1, 0, 1, 0},
            {1, 1, 0, 1, 0, 1, 0, 0, 0},
            {1, 1, 0, 1, 0, 0, 1, 0, 0},
            {1, 1, 0, 1, 0, 0, 0, 1, 0},
            {1, 1, 0, 0, 1, 0, 1, 0, 0},
            {1, 1, 0, 0, 1, 0, 0, 1, 0},
            {1, 1, 0, 0, 0, 1, 0, 1, 0},
            {1, 0, 1, 1, 0, 1, 0, 0, 0},
            {1, 0, 1, 1, 0, 0, 1, 0, 0},
            {1, 0, 1, 1, 0, 0, 0, 1, 0},
            {1, 0, 0, 1, 1, 0, 1, 0, 0},
            {1, 0, 0, 0, 1, 1, 0, 1, 0},
            {1, 0, 1, 0, 1, 1, 0, 0, 0},
            {1, 0, 1, 0, 0, 1, 1, 0, 0},
            {1, 0, 1, 0, 0, 0, 1, 1, 0},
            {1, 0, 0, 1, 0, 1, 1, 0, 0},
            {1, 0, 0, 0, 1, 0, 1, 1, 0},
            {1, 1, 0, 1, 1, 0, 1, 0, 0},
            {1, 1, 0, 1, 1, 0, 0, 1, 0},
            {1, 1, 0, 1, 0, 1, 1, 0, 0},
            {1, 1, 0, 1, 0, 0, 1, 1, 0},
            {1, 1, 0, 0, 1, 0, 1, 1, 0},
            {1, 1, 0, 0, 1, 1, 0, 1, 0},
            {1, 0, 1, 1, 0, 1, 1, 0, 0},
            {1, 0, 1, 1, 0, 0, 1, 1, 0},
            {1, 0, 0, 1, 1, 0, 1, 1, 0},
            {1, 0, 0, 1, 1, 1, 0, 1, 0},
            {1, 0, 0, 1, 0, 1, 1, 1, 0},
            {1, 1, 1, 0, 1, 0, 1, 0, 0},
            {1, 1, 1, 0, 1, 0, 0, 1, 0},
            {1, 1, 1, 0, 0, 1, 0, 1, 0},
            {1, 0, 1, 1, 0, 1, 1, 1, 0},
            {1, 0, 1, 1, 1, 0, 1, 1, 0},
            {1, 1, 0, 1, 0, 1, 1, 1, 0},
            {1, 0, 0, 1, 0, 0, 1, 1, 0},
            {1, 1, 1, 0, 1, 1, 0, 1, 0},
            {1, 1, 1, 0, 1, 0, 1, 1, 0},
            {1, 0, 0, 1, 1, 0, 0, 1, 0}
        };

        private readonly static int[] start_stop = { 1, 0, 1, 0, 1, 1, 1, 1, 0 };

        /// <summary>
        /// Zainicjuj
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            pictureBox.Image = new Bitmap(pictureBox.Width, pictureBox.Height, PixelFormat.Format24bppRgb);
            System.Drawing.SolidBrush whiteBrush = new System.Drawing.SolidBrush(System.Drawing.Color.White);
            System.Drawing.Graphics graphics = Graphics.FromImage(pictureBox.Image);
            graphics.FillRectangle(whiteBrush, new Rectangle(0, 0, pictureBox.Width, pictureBox.Height));
            pictureBox.Refresh();
            whiteBrush.Dispose();
            graphics.Dispose();
        }

        /// <summary>
        /// Sprawdza czy podany txt mozna zakodowac Code 93
        /// </summary>
        /// <param name="txt">tekst do zakodowania</param>
        /// <returns>true jezeli mozna zakodowac, false w przeciwnym razie</returns>
        public bool check(string txt)
        {
            for (int i = 0; i < txt.Length; i++)
            {
                if (checkLetter(txt[i]) == false)
                    return false;
            }
            return true;
        }

        /// <summary>
        /// Sprawdza czy podany znak mozna zakodowac Code 93
        /// </summary>
        /// <param name="ch">znak do zakodowania</param>
        /// <returns>true jezeli mozna zakodowac, false w przeciwnym razie</returns>
        private bool checkLetter(char ch)
        {
            for (int i = 0; i < code93signs.Length-4; i++)
            {
                if (code93signs[i] == ch)
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Zwraca wartosc danego znaku
        /// </summary>
        /// <param name="ch">znak</param>
        /// <returns>wartosc danego znaku</returns>
        private int getLetterValue(char ch)
        {
            for (int i = 0; i < code93signs.Length; i++)
                if (code93signs[i] == ch)
                    return i;

            return -1;
        }

        /// <summary>
        /// Oblicza cyfre kontrolna C
        /// </summary>
        /// <param name="code93">Ciag do zakodowania</param>
        /// <returns>cyfra kontrolna C</returns>
        private int checkDigitC(string code93)
        {
            int sum = 0;
            int w = 0;
 
            for (int i=code93.Length-1; i>=0; i--)
            {
                sum += (getLetterValue(code93[i]) * ((w % 20) + 1));
                w++;
            }
 
            return (sum % 47);
        }  
 
        /// <summary>
        /// Oblicza cyfre kontrolna C
        /// </summary>
        /// <param name="code93">Ciag do zakodowania</param>
        /// <returns>cyfra kontrolna C</returns>
        private int checkDigitK(string code93)
        {
            int sum = checkDigitC(code93);
            int w = 1;
 
            for (int i=code93.Length-1; i>=0; i--)
            {
                sum += (getLetterValue(code93[i]) * ((w % 15) + 1));
                w++;
            }

            return (sum % 47);
        }

        /// <summary>
        /// Generuj Code 93
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Generate_Click(object sender, EventArgs e)
        {
            //Zainicjuj narzedzia do rysowania
            System.Drawing.SolidBrush whiteBrush = new System.Drawing.SolidBrush(System.Drawing.Color.White);
            System.Drawing.SolidBrush blackBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Black);
            System.Drawing.Graphics graphics = Graphics.FromImage(pictureBox.Image);
            Font drawFont = new Font("Arial", 10);

            //Wyczysc obszar rysowania
            graphics.FillRectangle(whiteBrush, new Rectangle(0, 0, pictureBox.Width, pictureBox.Height));

            //Sprawdz czy podany ciag mozna zakodowac
            string code = Code.Text;
            if (check(code) == false)
            {
                MessageBox.Show("Nieprawidłowe znaki!", "Bład", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //Wygeneruj i narysuj kod
            int[] bars = generateBars(code);
            for (int i = 0; i < bars.Length; i++)
            {
                //Sprawdz czy pasek bialy czy czarny (rysujemy tylko czarne, bo tlo jest biale)
                if (bars[i] == 1)
                {
                    graphics.FillRectangle(blackBrush, 10+2*i, 10, 2, 90);
                }
            }

            //Wypisz kodowany ciag pod kodem kreskowym
            for (int i = 0; i < code.Length; i++)
            {
                graphics.DrawString(code.Substring(i, 1), drawFont, blackBrush, i * 18 + 35, 102);
            }

            //Odswiez
            pictureBox.Refresh();

            //Zwolinij zasoby
            whiteBrush.Dispose();
            blackBrush.Dispose();
            graphics.Dispose();
        }

        /// <summary>
        /// Generuje kod kreskowy
        /// </summary>
        /// <param name="code">string do zakodowania kodem kreskowym</param>
        /// <returns>kod kreskowy - 1 pasek czarny, 0 - pasek bialy</returns>
        private int[] generateBars(string code)
        {
            int[] bars = new int[9 * (code.Length+4) + 1];

            //Dodaj znak start/stop
            for (int j = 0; j < 9; j++)
            {
                bars[j] = start_stop[j];
            }

            //Dodaj kolejne znaki
            for (int i = 0; i < code.Length; i++)
            {
                int letterValue = getLetterValue(code[i]);
                for (int j = 0; j < 9; j++)
                {
                    bars[i * 9 + 9 + j] = code93bars[letterValue, j];
                }
            }

            //Dodaj cyfre kontrolna C
            int checkDigit = checkDigitC(code);
            for (int j = 0; j < 9; j++)
            {
                bars[(code.Length + 1) * 9 + j] = code93bars[checkDigit, j];
            }

            //Dodaj cyfre kontrolna K
            checkDigit = checkDigitK(code);
            for (int j = 0; j < 9; j++)
            {
                bars[(code.Length + 2) * 9 + j] = code93bars[checkDigit, j];
            }

            //Dodaj znak start/stop
            for (int j = 0; j < 9; j++)
            {
                bars[(code.Length + 3) * 9 + j] = start_stop[j];
            }

            //Dodaj koncowa kreske
            bars[(code.Length + 4) * 9] = 1;
            
            return bars;
        }
    }
}
